# Binance Connector Ruby

A thin layer for Binance API in Ruby


## Installation

Add this line to your application's Gemfile:

```ruby
gem 'binance-connector-ruby'
```

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install binance-connector-ruby

## Usage

```ruby

require 'binance'

# create a new client instance
client = Binance::Spot.new

# send request to query server time
puts client.time

# send request to query BTCUSDT ticker
puts client.ticker_24hr(symbol: 'BTCUSDT')

# create client instance for USER_DATA
client = Binance::Spot.new(key: key, secret: secret)

# send request to get account information
puts client.account

# place an order
response = client.new_order(symbol: 'BNBUSDT', side: 'BUY', price: 20, quantity: 1, type: 'LIMIT', timeInForce: 'GTC')

# cancel an order
response = client.cancel_order(symbol: 'BNBUSDT', orderId: '111')
```

### Key and Secret

The Key and secret can be setup from:
- The environment `BINANCE_PUBLIC_API_KEY` and `BINANCE_PRIVATE_SECRET`
- or provide during initializing the client

If both provided, use the key and secret from environment.

```ruby

ENV['BINANCE_PUBLIC_API_KEY']=xxxx
ENV['BINANCE_PRIVATE_SECRET']=xxxx
client = Binance::Spot.new
puts client.account
```

### Testnet

The [spot testnet](https://testnet.binance.vision/) is available. In order to test on testnet:

```ruby
client = Binance::Spot.new(base_url: 'https://testnet.binance.vision')
puts client.time
```

Without providing the base url, this connector works on `api.binance.com` by default.

It's recommended to allow changing the base url even in production.

### RecvWindow

From Binance API, recvWindow is available for all endpoints require signature. By default, it's 5000ms.
You are allowed to set this parameter to any value less than 60000, number beyond this limit will receive error from Binance server.
```ruby

response = client.account(recvWindow: 2_000)

```

### Optional parameters

For the optional parameters in the endpoint, pass exactly the field name from API document into method.
e.g

```ruby
client = Binance::Spot.new

# correct
response = client.cancel_order(symbol: 'BNBUSDT', orderId: 25)

# this is incorrect
response = client.cancel_order(symbol: 'BNBUSDT', order_id: 25)
```

### Timeout
To specific timeout time,
```ruby
# in second
client = Binance::Spot.new(timeout: 2)
```
if not set, there is no timeout

### Display meta info
Binance API server returns weight usage in the header of each response. This is very useful to indentify the current usage.
To reveal this value, simpily intial the client with `show_weight_usage: true` as:
```ruby
client = Binance::Spot.new(show_weight_usage: true)

puts client.time

```
then returns will be like

```ruby
{:data=>{:serverTime=>1589860878546}, :weight_usage=>{"x-mbx-used-weight"=>"1", "x-mbx-used-weight-1m"=>"1"}}
```

It's also able to print out all headers, which may be very helpful for debug:
```ruby
client = Binance::Spot.new(show_header: true)

puts client.time
```
then returns will be like

```ruby
{:data=>{:serverTime=>1589863539220}, :header=>{"content-type"=>"application/json;charset=utf-8",...}}
```

### Display logs

To debug, pass the logger in the client.
```ruby
logger = Logger.new(STDOUT)
client = Binance::Spot.new(logger: logger)
puts client.time
```

### Exception

There are 3 types of exceptions:
- `Binance::RequiredParams`
  - When missing required param
  - Not sending out the request
- `Binance::ClientError`
  - When API server returning 4XX
  - response is return in the error message
  - sending out the request
- `Binance::ServerError`
  - When API server returning 5XX
  - sending out the request

They all inherit from `Binance::Error`

```ruby
begin
  response = client.time
rescue Binance::ClientError => e
  e.response[:status]
  e.response[:headers]
  e.response[:body]
end

```

## License
MIT
